using System.Security.Cryptography;
using System.Text;
using FitConnect.Models.Api.Metadata;

namespace FitConnect.Models;

public class Attachment {
    public Attachment() {
    }

    /// <summary>
    ///     Attachment for a submission.
    /// </summary>
    /// <param name="filename">File name to load</param>
    /// <param name="description">Description of the attachment</param>
    public Attachment(string filename, string description) {
        var file = new FileInfo(filename);
        Filename = filename;
        Content = File.ReadAllBytes(filename);
        var extension = file.Extension[1..].ToLower();
        MimeType = "application/" + extension;
        Description = description;


        Signature = new AttachmentSignature {
            DetachedSignature = true,
            SignatureFormat = extension switch {
                "pdf" => SignatureFormat.Pdf,
                "xml" => SignatureFormat.Xml,
                "json" => SignatureFormat.Json,
                _ => throw new ArgumentOutOfRangeException()
            }
        };
    }

    public string Id { get; } = Guid.NewGuid().ToString();

    public byte[]? Content { get; init; }

    public string? Hash => CalculateHash();

    public string? Filename { get; init; }

    public string? Purpose { get; init; } = "attachment";

    public string? MimeType { get; init; }

    public string? Description { get; init; }

    public AttachmentSignature? Signature { get; }

    private string CalculateHash() {
        return ByteToHexString(SHA512.Create().ComputeHash(Content));
    }

    private static string ByteToHexString(IEnumerable<byte> data) {
        var sb = new StringBuilder();
        foreach (var b in data) sb.Append(b.ToString("x2"));

        return sb.ToString();
    }
}
